'************************************************
'* NTSC 128 x 96 - 8bit Bitmap R3G3B2 Composite *
'* Driver by Jim Bagley                         *
'************************************************

' P0 = YIQ

CON

_clkmode        = xinput
_xinfreq        = 60_000_000

BITMAPBUFFER	= $5000
BORDERCOLOUR	= $4FFC	'RGB 32bit
VSYNCFLAG	= $4FF8 '

s               = 46                    'scales DAC output (s = 0..128)
r               = s * 78 / 128          'adjusts for modulator expansion

lntsc           = 3640                  'NTSC colorburst cycles per line * 16
sntsc           = 624                   'NTSC colorburst cycles per sync * 16


DAT             org
		JMP     #task0          'task 0 begins here when the cog starts (this JMP takes 4 clocks)
		JMP     #task1          'task 1 begins here after task 0 executes SETTASK (this JMP takes 1 clock)
		JMP     #task2          'task 2 begins here after task 0 executes SETTASK (this JMP takes 1 clock)
		JMP     #task3          'task 3 begins here after task 0 executes SETTASK (this JMP takes 1 clock)
task0		SETTASK tasks

'
'       Setup
'
		setctra #%00111         'set ctra for PLL*8
		setfrqa frqa_           'set frqa

		setvid  #%1110          'set vid for composite output
		setvidy vidy_           'set yiq coefficients
		setvidi vidi_
		setvidq vidq_

		cfgdac0 #%11            'set bottom dac to video mode (YIQ)
		cfgpins pinmask,pindacs 'set P0 for dac output

		wrlong	zero,bordercolptr
'
' Field loop --non interlaced 60Hz NTSC frame
'

field		mov     vidlp,#34'-13   'top blanks
		call    #blank
		SETTASK tasks

		setptra bitmapptr       'Get ready to scan pixels
		mov     ycount,#0
		mov     ypix,#0
		mov     vidlp,#192      'set visible lines
line		call    #hsync          'do horizontal sync

		SETTASK tasks0
		reps	#32,#2
		setspa	#0
		rdlongc	tmp1,ptra++
		pusha	tmp1
		SETTASK tasks

		rdlong  bordercol,bordercolptr
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bo,bordercol  'black left border

		polvid	wc
	if_nc	jmp	#$-1
		waitvid	mode,#0

		rdlong  bordercol,bordercolptr
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bo,bordercol  'black right border

		add     ycount,#1
		incmod	ypix,#1 wz
	if_z	subptra #128

		djnz    vidlp,#line     'Do all the lines

		wrlong  one,framesync   'Set framesync

		mov     vidlp,#28'-(32-13)      'bottom blanks
		call    #blank

synch		mov     vidlp,#6        'high vertical syncs
		call    #vsynch
		mov     vidlp,#6        'low vertical syncs
		call    #vsyncl
		mov     vidlp,#6        'high vertical syncs
		call    #vsynch
		jmp     #field          'field loop
'
'  Subs
'
blank		call    #hsync          'blank lines
		SETTASK tasks
		rdlong  bordercol,bordercolptr
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bl,bordercol  'black right border
		djnz    vidlp,#blank
blank_ret	ret


hsync		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bs,c_sh       'horizontal sync
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_sn,c_sl
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bc,c_sh
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_cb,c_cb
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bv,c_sh
hsync_ret	ret                    'Pipeline... get ready to return

vsynch		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bs,c_sh       'vertical sync high
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_hs,c_sl
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_hr,c_sh
		djnz    vidlp,#vsynch
vsynch_ret	ret


vsyncl		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_bs,c_sh       'vertical sync low
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_ls,c_sl
		polvid	wc
	if_nc	jmp	#$-1
		waitvid v_lr,c_sh
		djnz    vidlp,#vsyncl
vsyncl_ret	ret

'
'Begin COG Data....
'

bordercol       long    0
bordercolptr	long	BORDERCOLOUR         'pointing to the bordercol long in hubram for other cogs to change it
bitmapptr       long    BITMAPBUFFER                  'start of bitmap'
framesync       long    VSYNCFLAG
one             long    1
zero		long	0

ycount          long    0
ypix            long    0
tmp1            long    0
tmp2            long    0
tmp3            long    0
clp             long    0

vidlp           long    0                       'Renamed x to vidlp to remove x as it may be used in another cog program
vidrgb          long    0

frqa_           long    $1E8B_A2A7              '57.272720 MHz (colorburst * 16)

vidy_           long    (110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_           long    (100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_           long                  128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)

pinmask         long    %1                      'pin mask for P0
pindacs         long    %1111_101100000         'pin configuration for video DAC

                '       color base      mode            clocks/pixel    clocks/waitvid
mode            long    $00 << 24 +     %1100 << 20 +  18 << 13 +   	128*18  'mode $f here + 9 = 256 pixels

v_bo            long    (lntsc-sntsc-128*18)/2   'left/right border

v_bs            long    100                     'before sync            horizontal sync
v_sn            long    269                     'sync
v_bc            long    50                      'before colorburst
v_cb            long    144                     'colorburst
v_bv            long    sntsc-144-50-269-100    'before visible

v_hs            long    135                     'high sync              high vertical sync
v_hr            long    lntsc/2-135-100         'high sync remainder

v_ls            long    1564                    'low sync               low vertical sync
v_lr            long    lntsc/2-1564-100        'low sync remainder

v_bl            long    lntsc-sntsc             'blank visible

c_sh            long    $1_000000               'sync high
c_sl            long    $2_000000               'sync low
c_cb            long    $1_707000               'colorburst reference color

'-------------------------------------------------------------

tasks	long	%%0111_0222_0333_0123
tasks0	long	0

'-------------------------------------------------------------

task1
	notp	#32
	rdlong	bordercol,bordercolptr
	jmp	#task1
task2
	notp	#33
	rdlong	bordercol,bordercolptr
	jmp	#task2
task3
	notp	#34
	rdlong	bordercol,bordercolptr
	jmp	#task3

'-------------------------------------------------------------
